@@ -1,20 +1,19 @@
-package HTTP::Server::Simple;
 use strict;
 use warnings;
+
+package HTTP::Server::Simple;
 use FileHandle;
 use Socket;
 use Carp;
 use URI::Escape;
 
 use vars qw($VERSION $bad_request_doc);
-$VERSION = '0.28_01';
-
+$VERSION = '0.38_01';
 
 =head1 NAME
 
 HTTP::Server::Simple - Lightweight HTTP server
 
-
 =head1 SYNOPSIS
 
  use warnings;
@@ -43,14 +42,13 @@ module (see L<HTTP::Server::Simple::CGI>);
 
 =head1 DESCRIPTION
 
-This is a simple standalone HTTP server. By default, it doesn't thread 
-or fork.
-
-It does, however, act as a simple frontend which can be used 
+This is a simple standalone HTTP server. By default, it doesn't thread
+or fork. It does, however, act as a simple frontend which can be used
 to build a standalone web-based application or turn a CGI into one.
 
-(It's possible to use Net::Server to get threading, forking,
-preforking and so on. Autrijus Tang wrote the functionality and owes docs for that ;)
+It is possible to use L<Net::Server> classes to create forking,
+pre-forking, and other types of more complicated servers; see
+L</net_server>.
 
 By default, the server traps a few signals:
 
@@ -69,14 +67,68 @@ HUP to all running processes spawned by your app (e.g. by "kill -HUP <script>")
 
 If the server detects a broken pipe while writing output to the client, 
 it ignores the signal. Otherwise, a client closing the connection early 
-could kill the server
+could kill the server.
 
 =back
 
+=head1 EXAMPLE
+ 
+ #!/usr/bin/perl
+ {
+ package MyWebServer;
+ 
+ use HTTP::Server::Simple::CGI;
+ use base qw(HTTP::Server::Simple::CGI);
+ 
+ my %dispatch = (
+     '/hello' => \&resp_hello,
+     # ...
+ );
+ 
+ sub handle_request {
+     my $self = shift;
+     my $cgi  = shift;
+   
+     my $path = $cgi->path_info();
+     my $handler = $dispatch{$path};
+ 
+     if (ref($handler) eq "CODE") {
+         print "HTTP/1.0 200 OK\r\n";
+         $handler->($cgi);
+         
+     } else {
+         print "HTTP/1.0 404 Not found\r\n";
+         print $cgi->header,
+               $cgi->start_html('Not found'),
+               $cgi->h1('Not found'),
+               $cgi->end_html;
+     }
+ }
+ 
+ sub resp_hello {
+     my $cgi  = shift;   # CGI.pm object
+     return if !ref $cgi;
+     
+     my $who = $cgi->param('name');
+     
+     print $cgi->header,
+           $cgi->start_html("Hello"),
+           $cgi->h1("Hello $who!"),
+           $cgi->end_html;
+ }
+ 
+ } 
+ 
+ # start the server on port 8080
+ my $pid = MyWebServer->new(8080)->background();
+ print "Use 'kill $pid' to stop server.\n";
+
+=head1 METHODS
+
 =head2 HTTP::Server::Simple->new($port)
 
 API call to start a new server.  Does not actually start listening
-until you call C<-E<gt>run()>.
+until you call C<-E<gt>run()>.  If omitted, C<$port> defaults to 8080.
 
 =cut
 
@@ -93,19 +145,14 @@ sub new {
     bless( $self, $class );
     $self->port( $port || '8080' );
 
-
-
     return $self;
 }
 
 
 =head2 lookup_localhost
 
-Looks up the local host's hostname and IP address.
-
-Stuffs them into
-
-$self->{'localname'} and $self->{'localaddr'}
+Looks up the local host's IP address, and returns it.  For most hosts,
+this is C<127.0.0.1>.
 
 =cut
 
@@ -119,8 +166,6 @@ sub lookup_localhost {
 }
 
 
-
-
 =head2 port [NUMBER]
 
 Takes an optional port number for this server to listen on.
@@ -152,30 +197,43 @@ sub host {
 
 }
 
-=head2 background
+=head2 background [ARGUMENTS]
 
-Run the server in the background. returns pid.
+Runs the server in the background, and returns the process ID of the
+started process.  Any arguments will be passed through to L</run>.
 
 =cut
 
 sub background {
     my $self  = shift;
+    require File::Temp;
+    my ($fh, $filename) = File::Temp::tempfile();
+    unlink($filename);
     my $child = fork;
-    die "Can't fork: $!" unless defined($child);
-    return $child if $child;
+    croak "Can't fork: $!" unless defined($child);
+    if ($child) {
+        while (eof($fh)) {
+            select(undef, undef, undef, 0.1);
+            seek($fh, 0, 0);
+        }
+        return $child;
+    }
 
     if ( $^O !~ /MSWin32/ ) {
         require POSIX;
         POSIX::setsid()
-            or die "Can't start a new session: $!";
+            or croak "Can't start a new session: $!";
     }
-    $self->run();
+    $self->{after_setup} = sub { print {$fh} 1; close $fh };
+    $self->run(@_);
 }
 
-=head2 run
+=head2 run [ARGUMENTS]
 
 Run the server.  If all goes well, this won't ever return, but it will
-start listening for http requests.
+start listening for C<HTTP> requests.  Any arguments passed to this
+will be passed on to the underlying L<Net::Server> implementation, if
+one is used (see L</net_server>).
 
 =cut
 
@@ -201,7 +259,14 @@ sub run {
     if ($server) {
         require join( '/', split /::/, $server ) . '.pm';
         *{"$pkg\::ISA"} = [$server];
-        $self->print_banner;
+
+        # clear the environment before every request
+        require HTTP::Server::Simple::CGI;
+        *{"$pkg\::post_accept"} = sub {
+            HTTP::Server::Simple::CGI::Environment->setup_environment;
+            # $self->SUPER::post_accept uses the wrong super package
+            $server->can('post_accept')->(@_);
+        };
     }
     else {
         $self->setup_listener;
@@ -211,13 +276,13 @@ sub run {
 
     local $SIG{HUP} = sub { $SERVER_SHOULD_RUN = 0; };
 
-    $pkg->run( port => $self->port );
+    $pkg->run( port => $self->port, @_ );
 }
 
 =head2 net_server
 
-User-overridable method. If you set it to a C<Net::Server> subclass,
-that subclass is used for the C<run> method.  Otherwise, a minimal 
+User-overridable method. If you set it to a L<Net::Server> subclass,
+that subclass is used for the C<run> method.  Otherwise, a minimal
 implementation is used as default.
 
 =cut
@@ -342,13 +407,9 @@ sub _process_request {
         $self->post_setup_hook if $self->can("post_setup_hook");
 
         $self->handler;
-        }
+    }
 }
 
-
-
-
-
 =head2 stdio_handle [FILEHANDLE]
 
 When called with an argument, sets the socket to the server to that arg.
@@ -368,9 +429,9 @@ sub stdio_handle {
 
 =head2 stdin_handle
 
-Returns a filehandle used for input from the client.  By default, 
-returns whatever was set with C<stdio_handle>, but a subclass
-could do something interesting here (see L<HTTP::Server::Simple::Logger>).
+Returns a filehandle used for input from the client.  By default,
+returns whatever was set with C<stdio_handle>, but a subclass could do
+something interesting here.
 
 =cut
 
@@ -383,7 +444,7 @@ sub stdin_handle {
 
 Returns a filehandle used for output to the client.  By default, 
 returns whatever was set with C<stdio_handle>, but a subclass
-could do something interesting here (see L<HTTP::Server::Simple::Logger>).
+could do something interesting here.
 
 =cut
 
@@ -411,7 +472,7 @@ sub handler {
         croak "do not call " . ref($self) . "::SUPER->handler";
     }
     else {
-        die "handler called out of context";
+        croak "handler called out of context";
     }
 }
 
@@ -476,17 +537,34 @@ sub headers {
     my $headers = shift;
 
     my $can_header = $self->can("header");
+    return unless $can_header;
     while ( my ( $header, $value ) = splice @$headers, 0, 2 ) {
-        if ($can_header) {
-            $self->header( $header => $value );
-        }
+        $self->header( $header => $value );
     }
 }
 
 =head2 accept_hook
 
 If defined by a sub-class, this method is called directly after an
-accept happens.
+accept happens.  An accept_hook to add SSL support might look like this:
+
+    sub accept_hook {
+        my $self = shift;
+        my $fh   = $self->stdio_handle;
+
+        $self->SUPER::accept_hook(@_);
+
+        my $newfh =
+        IO::Socket::SSL->start_SSL( $fh, 
+            SSL_server    => 1,
+            SSL_use_cert  => 1,
+            SSL_cert_file => 'myserver.crt',
+            SSL_key_file  => 'myserver.key',
+        )
+        or warn "problem setting up SSL socket: " . IO::Socket::SSL::errstr();
+
+        $self->stdio_handle($newfh) if $newfh;
+    }
 
 =head2 post_setup_hook
 
@@ -506,7 +584,7 @@ yourself in subclasses.
 sub print_banner {
     my $self = shift;
 
-    print(    __PACKAGE__
+    print( ref($self) 
             . ": You can connect to your server at "
             . "http://localhost:"
             . $self->port
@@ -516,11 +594,8 @@ sub print_banner {
 
 =head2 parse_request
 
-Parse the HTTP request line.
-
-Returns three values, the request method, request URI and the protocol
-Sub-classed versions of this should return three values - request
-method, request URI and proto
+Parse the HTTP request line.  Returns three values, the request
+method, request URI and the protocol.
 
 =cut
 
@@ -544,13 +619,9 @@ sub parse_request {
 
 =head2 parse_headers
 
-Parse incoming HTTP headers from STDIN.
-
-Remember, this is a B<simple> HTTP server, so nothing intelligent is
-done with them C<:-)>.
-
-This should return an ARRAY ref of C<(header =E<gt> value)> pairs
-inside the array.
+Parses incoming HTTP headers from STDIN, and returns an arrayref of
+C<(header =E<gt> value)> pairs.  See L</headers> for possibilities on
+how to inspect headers.
 
 =cut
 
@@ -563,7 +634,7 @@ sub parse_headers {
     while ( sysread( STDIN, my $buff, 1 ) ) {
         if ( $buff eq "\n" ) {
             $chunk =~ s/[\r\l\n\s]+$//;
-            if ( $chunk =~ /^([\w\-]+): (.+)/i ) {
+            if ( $chunk =~ /^([^()<>\@,;:\\"\/\[\]?={} \t]+):\s*(.*)/i ) {
                 push @headers, $1 => $2;
             }
             last if ( $chunk =~ /^$/ );
@@ -585,8 +656,7 @@ sub setup_listener {
     my $self = shift;
 
     my $tcp = getprotobyname('tcp');
-
-    socket( HTTPDaemon, PF_INET, SOCK_STREAM, $tcp ) or die "socket: $!";
+    socket( HTTPDaemon, PF_INET, SOCK_STREAM, $tcp ) or croak "socket: $!";
     setsockopt( HTTPDaemon, SOL_SOCKET, SO_REUSEADDR, pack( "l", 1 ) )
         or warn "setsockopt: $!";
     bind( HTTPDaemon,
@@ -598,15 +668,16 @@ sub setup_listener {
             )
         )
         )
-        or die "bind: $!";
-    listen( HTTPDaemon, SOMAXCONN ) or die "listen: $!";
-
+        or croak "bind to @{[$self->host||'*']}:@{[$self->port]}: $!";
+    listen( HTTPDaemon, SOMAXCONN ) or croak "listen: $!";
+    $self->{after_setup} && $self->{after_setup}->();
 }
 
 
 =head2 after_setup_listener
 
-This method is called immediately after setup_listener. It's here just for you to override.
+This method is called immediately after setup_listener. It's here just
+for you to override.
 
 =cut
 
@@ -633,7 +704,9 @@ sub bad_request {
 =head2 valid_http_method($method)
 
 Given a candidate HTTP method in $method, determine if it is valid.
-Override if, for example, you'd like to do some WebDAV.
+Override if, for example, you'd like to do some WebDAV.  The default
+implementation only accepts C<GET>, C<POST>, C<HEAD>, C<PUT>, and
+C<DELETE>.
 
 =cut 
 
@@ -645,7 +718,7 @@ sub valid_http_method {
 
 =head1 AUTHOR
 
-Copyright (c) 2004-2006 Jesse Vincent, <jesse@bestpractical.com>.
+Copyright (c) 2004-2008 Jesse Vincent, <jesse@bestpractical.com>.
 All rights reserved.
 
 Marcus Ramberg <drave@thefeed.no> contributed tests, cleanup, etc
@@ -653,6 +726,8 @@ Marcus Ramberg <drave@thefeed.no> contributed tests, cleanup, etc
 Sam Vilain, <samv@cpan.org> contributed the CGI.pm split-out and
 header/setup API.
 
+Example section by almut on perlmonks, suggested by Mark Fuller.
+
 =head1 BUGS
 
 There certainly are some. Please report them via rt.cpan.org
